==> fichu-0.1.13.tar.gz: fichu-0.1.13/src/server/message_handler/mod.rs <==
mod completion;
mod diagnostic;
mod formatting;
mod hovering;
use std::process::exit;

use completion::handel_completion_request;
use hovering::handle_hover_request;
use log::{debug, error, info, warn};

pub use diagnostic::*;
pub use formatting::format_raw;

use crate::{
    lsp::{
        textdocument::TextDocumentItem, CompletionRequest, Diagnostic, DiagnosticRequest,
        DiagnosticResponse, DidChangeTextDocumentNotification, DidOpenTextDocumentNotification,
        FormattingRequest, HoverRequest, InitializeRequest, InitializeResonse, ShutdownResponse,
    },
    rpc::{self, RequestMessage},
    server::{ServerState, ServerStatus},
};

use self::formatting::handle_format_request;

use super::{configuration::Settings, Server};

pub fn dispatch(server: &mut Server, bytes: &Vec<u8>) -> Option<String> {
    if let Ok(message) = rpc::decode_message(bytes) {
        match message.method.as_str() {
            "initialize" => match serde_json::from_slice::<InitializeRequest>(bytes) {
                Ok(initialize_request) => {
                    info!(
                        "Connected to: {} {}",
                        initialize_request.params.client_info.name,
                        initialize_request
                            .params
                            .client_info
                            .version
                            .unwrap_or("no version specified".to_string())
                    );
                    let initialize_response =
                        InitializeResonse::new(initialize_request.base.id, server);
                    return Some(serde_json::to_string(&initialize_response).unwrap());
                }
                Err(error) => {
                    error!("Could not parse initialize request: {:?}", error);
                    return None;
                }
            },
            "initialized" => {
                info!("initialization completed");
                server.state.status = ServerStatus::Running;
                return None;
            }
            "shutdown" => match serde_json::from_slice::<RequestMessage>(bytes) {
                Ok(shutdown_request) => {
                    info!("recieved shutdown request, preparing to shut down");
                    let response = ShutdownResponse::new(shutdown_request.id);
                    server.state.status = ServerStatus::ShuttingDown;
                    return Some(serde_json::to_string(&response).unwrap());
                }
                Err(error) => {
                    error!("Could not parse shutdown request: {:?}", error);
                    return None;
                }
            },
            "exit" => {
                info!("recieved exit notification, shutting down!");
                exit(0);
            }
            "textDocument/didOpen" => {
                match serde_json::from_slice::<DidOpenTextDocumentNotification>(bytes) {
                    Ok(did_open_notification) => {
                        debug!(
                            "opened text document: \"{}\"\n{}",
                            did_open_notification.params.text_document.uri,
                            did_open_notification.params.text_document.text
                        );
                        let text_document: TextDocumentItem =
                            did_open_notification.get_text_document();
                        server.state.add_document(text_document);
                        return None;
                    }
                    Err(error) => {
                        error!("Could not parse textDocument/didOpen request: {:?}", error);
                        return None;
                    }
                }
            }
            "textDocument/didChange" => {
                match serde_json::from_slice::<DidChangeTextDocumentNotification>(bytes) {
                    Ok(did_change_notification) => {
                        debug!(
                            "text document changed: {}",
                            did_change_notification.params.text_document.base.uri
                        );
                        server.state.change_document(
                            did_change_notification.params.text_document.base.uri,
                            did_change_notification.params.content_changes,
                        );

                        return None;
                    }
                    Err(error) => {
                        error!(
                            "Could not parse textDocument/didChange notification: {:?}",
                            error
                        );
                        return None;
                    }
                }
            }
            "textDocument/hover" => match serde_json::from_slice::<HoverRequest>(bytes) {
                Ok(hover_request) => {
                    debug!(
                        "recieved hover request for {} {}",
                        hover_request.get_document_uri(),
                        hover_request.get_position()
                    );
                    let response = handle_hover_request(&hover_request, &server.state);

                    return Some(serde_json::to_string(&response).unwrap());
                }
                Err(error) => {
                    error!("Could not parse textDocument/hover request: {:?}", error);
                    return None;
                }
            },
            "textDocument/completion" => match serde_json::from_slice::<CompletionRequest>(bytes) {
                Ok(completion_request) => {
                    debug!(
                        "Received completion request for {} {}",
                        completion_request.get_document_uri(),
                        completion_request.get_position()
                    );
                    let response = handel_completion_request(completion_request, &mut server.state);
                    return Some(serde_json::to_string(&response).unwrap());
                }
                Err(error) => {
                    error!(
                        "Could not parse textDocument/completion request: {:?}",
                        error
                    );
                    return None;
                }
            },
            "textDocument/formatting" => match serde_json::from_slice::<FormattingRequest>(bytes) {
                Ok(formatting_request) => {
                    let response = handle_format_request(
                        formatting_request,
                        &mut server.state,
                        &server.settings,
                    );
                    return Some(serde_json::to_string(&response).unwrap());
                }
                Err(error) => {
                    error!(
                        "Could not parse textDocument/formatting request: {:?}",
                        error
                    );
                    return None;
                }
            },
            "textDocument/diagnostic" => match serde_json::from_slice::<DiagnosticRequest>(bytes) {
                Ok(diagnostic_request) => {
                    let diagnostics: Vec<Diagnostic> = collect_diagnostics(
                        &server.state.analysis_state,
                        &diagnostic_request.params.text_document.uri,
                    )
                    .collect();
                    let resonse = DiagnosticResponse::new(diagnostic_request.base.id, diagnostics);
                    return Some(serde_json::to_string(&resonse).unwrap());
                }
                Err(error) => {
                    error!(
                        "Could not parse textDocument/diagnostic request: {:?}",
                        error
                    );
                    return None;
                }
            },
            unknown_method => {
                warn!(
                    "Received message with unknown method \"{}\": {:?}",
                    unknown_method,
                    String::from_utf8(bytes.to_vec()).unwrap()
                );
                return None;
            }
        };
    } else {
        error!("An error occured while parsing the request content");
        return None;
    }
}

==> fichu-0.1.14.tar.gz: fichu-0.1.14/src/server/message_handler/mod.rs <==
mod completion;
mod diagnostic;
mod formatting;
mod hovering;
use std::process::exit;

use completion::handel_completion_request;
use hovering::handle_hover_request;
use log::{debug, error, info, warn};

pub use diagnostic::*;
pub use formatting::format_raw;

use crate::server::lsp::ProgressNotification;

use self::formatting::handle_format_request;

use super::{
    lsp::{
        rpc::{self, RequestMessage},
        textdocument::TextDocumentItem,
        CompletionRequest, Diagnostic, DiagnosticRequest, DiagnosticResponse,
        DidChangeTextDocumentNotification, DidOpenTextDocumentNotification, FormattingRequest,
        HoverRequest, InitializeRequest, InitializeResonse, ShutdownResponse,
    },
    state::ServerStatus,
    Server,
};

pub fn dispatch(server: &mut Server, bytes: &Vec<u8>) -> Option<String> {
    if let Ok(message) = rpc::decode_message(bytes) {
        match message.method.as_str() {
            "initialize" => match serde_json::from_slice::<InitializeRequest>(bytes) {
                Ok(initialize_request) => {
                    info!(
                        "Connected to: {} {}",
                        initialize_request.params.client_info.name,
                        initialize_request
                            .params
                            .client_info
                            .version
                            .unwrap_or("no version specified".to_string())
                    );
                    info!(
                        "wdtoken: {:?}",
                        initialize_request.params.progress_params.work_done_token
                    );
                    if let Some(work_done_token) =
                        initialize_request.params.progress_params.work_done_token
                    {
                        let init_progress_begin_notification =
                            ProgressNotification::begin_notification(
                                work_done_token.clone(),
                                "initlializing fichu language server",
                                Some(false),
                                Some("init"),
                                Some(0),
                            );
                        server.send_message(
                            serde_json::to_string(&init_progress_begin_notification).unwrap(),
                        );

                        // TODO: implement server side requests
                        let progress_report_1 = ProgressNotification::report_notification(
                            work_done_token.clone(),
                            Some(false),
                            Some("testing availibility of endpoint"),
                            Some(30),
                        );
                        server.send_message(serde_json::to_string(&progress_report_1).unwrap());

                        let progress_report_2 = ProgressNotification::report_notification(
                            work_done_token.clone(),
                            Some(false),
                            Some("request prefixes from endpoint"),
                            Some(60),
                        );
                        server.send_message(serde_json::to_string(&progress_report_2).unwrap());

                        let init_progress_end_notification = ProgressNotification::end_notification(
                            work_done_token.clone(),
                            Some("fichu initialized"),
                        );

                        server.send_message(
                            serde_json::to_string(&init_progress_end_notification).unwrap(),
                        );
                    }
                    let initialize_response =
                        InitializeResonse::new(initialize_request.base.id, server);
                    return Some(serde_json::to_string(&initialize_response).unwrap());
                }
                Err(error) => {
                    error!("Could not parse initialize request: {:?}", error);
                    return None;
                }
            },
            "initialized" => {
                info!("initialization completed");
                server.state.status = ServerStatus::Running;
                return None;
            }
            "shutdown" => match serde_json::from_slice::<RequestMessage>(bytes) {
                Ok(shutdown_request) => {
                    info!("recieved shutdown request, preparing to shut down");
                    let response = ShutdownResponse::new(shutdown_request.id);
                    server.state.status = ServerStatus::ShuttingDown;
                    return Some(serde_json::to_string(&response).unwrap());
                }
                Err(error) => {
                    error!("Could not parse shutdown request: {:?}", error);
                    return None;
                }
            },
            "exit" => {
                info!("recieved exit notification, shutting down!");
                exit(0);
            }
            "textDocument/didOpen" => {
                match serde_json::from_slice::<DidOpenTextDocumentNotification>(bytes) {
                    Ok(did_open_notification) => {
                        debug!(
                            "opened text document: \"{}\"\n{}",
                            did_open_notification.params.text_document.uri,
                            did_open_notification.params.text_document.text
                        );
                        let text_document: TextDocumentItem =
                            did_open_notification.get_text_document();
                        server.state.add_document(text_document);
                        return None;
                    }
                    Err(error) => {
                        error!("Could not parse textDocument/didOpen request: {:?}", error);
                        return None;
                    }
                }
            }
            "textDocument/didChange" => {
                match serde_json::from_slice::<DidChangeTextDocumentNotification>(bytes) {
                    Ok(did_change_notification) => {
                        debug!(
                            "text document changed: {}",
                            did_change_notification.params.text_document.base.uri
                        );
                        server.state.change_document(
                            did_change_notification.params.text_document.base.uri,
                            did_change_notification.params.content_changes,
                        );

                        return None;
                    }
                    Err(error) => {
                        error!(
                            "Could not parse textDocument/didChange notification: {:?}",
                            error
                        );
                        return None;
                    }
                }
            }
            "textDocument/hover" => match serde_json::from_slice::<HoverRequest>(bytes) {
                Ok(hover_request) => {
                    debug!(
                        "recieved hover request for {} {}",
                        hover_request.get_document_uri(),
                        hover_request.get_position()
                    );
                    let response = handle_hover_request(&hover_request, &server.state);

                    return Some(serde_json::to_string(&response).unwrap());
                }
                Err(error) => {
                    error!("Could not parse textDocument/hover request: {:?}", error);
                    return None;
                }
            },
            "textDocument/completion" => match serde_json::from_slice::<CompletionRequest>(bytes) {
                Ok(completion_request) => {
                    debug!(
                        "Received completion request for {} {}",
                        completion_request.get_document_uri(),
                        completion_request.get_position()
                    );
                    let response = handel_completion_request(completion_request, &mut server.state);
                    return Some(serde_json::to_string(&response).unwrap());
                }
                Err(error) => {
                    error!(
                        "Could not parse textDocument/completion request: {:?}",
                        error
                    );
                    return None;
                }
            },
            "textDocument/formatting" => match serde_json::from_slice::<FormattingRequest>(bytes) {
                Ok(formatting_request) => {
                    let response = handle_format_request(
                        formatting_request,
                        &mut server.state,
                        &server.settings,
                    );
                    return Some(serde_json::to_string(&response).unwrap());
                }
                Err(error) => {
                    error!(
                        "Could not parse textDocument/formatting request: {:?}",
                        error
                    );
                    return None;
                }
            },
            "textDocument/diagnostic" => match serde_json::from_slice::<DiagnosticRequest>(bytes) {
                Ok(diagnostic_request) => {
                    let diagnostics: Vec<Diagnostic> = collect_diagnostics(
                        &server.state,
                        &diagnostic_request.params.text_document.uri,
                    )
                    .collect();
                    let resonse = DiagnosticResponse::new(diagnostic_request.base.id, diagnostics);
                    return Some(serde_json::to_string(&resonse).unwrap());
                }
                Err(error) => {
                    error!(
                        "Could not parse textDocument/diagnostic request: {:?}",
                        error
                    );
                    return None;
                }
            },
            unknown_method => {
                warn!(
                    "Received message with unknown method \"{}\": {:?}",
                    unknown_method,
                    String::from_utf8(bytes.to_vec()).unwrap()
                );
                return None;
            }
        };
    } else {
        error!("An error occured while parsing the request content");
        return None;
    }
}
